@@ -261,11 +261,16 @@ def copy(source=None, target=None, id=None, safe_id=False):
 
     new_id = copy_info[0]['new_id']
     if id:
-        return rename(obj=target[new_id], new_id=id, safe_id=safe_id)
+        if not safe_id and id in target:
+            msg = "Duplicate ID '{0}' in '{1}' for '{2}'"
+            raise InvalidParameterError(msg.format(id, target, source))
+        else:
+            return rename(obj=target[new_id], new_id=id, safe_id=safe_id)
     else:
         return target[new_id]
 
 
+@mutually_exclusive_parameters('obj', 'objects')
 @at_least_one_of('obj', 'objects')
 def delete(obj=None, objects=None, check_linkintegrity=True):
     """Delete the object(s).
@@ -284,6 +289,8 @@ def delete(obj=None, objects=None, check_linkintegrity=True):
 
     :Example: :ref:`content_delete_example`
     """
+    objects = objects or [obj]
+
     if check_linkintegrity and NEW_LINKINTEGRITY:
         site = portal.get()
         linkintegrity_view = get_view(
@@ -291,30 +298,24 @@ def delete(obj=None, objects=None, check_linkintegrity=True):
             context=site,
             request=site.REQUEST)
 
-        objects_to_check = objects or [obj]
-        breaches = linkintegrity_view.get_breaches(objects_to_check)
+        # look for breaches and manually raise a exception
+        breaches = linkintegrity_view.get_breaches(objects)
         if breaches:
             raise LinkIntegrityNotificationException(
                 'Linkintegrity-breaches: {0}'.format(breaches)
             )
 
-    if obj is not None:
-        if check_linkintegrity or (
-                not check_linkintegrity and NEW_LINKINTEGRITY):
-            obj.aq_parent.manage_delObjects([obj.getId()])
-        else:
-            # old: we have to explicitly ignore the exception
+    for obj_ in objects:
+        if not check_linkintegrity and not NEW_LINKINTEGRITY:
+            # old style ignoring breaches:
+            # we have to explicitly ignore the exception
             try:
-                obj.aq_parent.manage_delObjects([obj.getId()])
+                obj_.aq_parent.manage_delObjects([obj_.getId()])
             except LinkIntegrityNotificationException:
                 pass
-
-    else:
-        check = False
-        if check_linkintegrity and not NEW_LINKINTEGRITY:
-            check = True
-        for obj in objects:
-            delete(obj=obj, check_linkintegrity=check)
+        else:
+            # All other cases
+            obj_.aq_parent.manage_delObjects([obj_.getId()])
 
 
 @required_parameters('obj')
@@ -484,6 +485,32 @@ def transition(obj=None, transition=None, to_state=None, **kwargs):
             )
 
 
+@required_parameters('obj')
+def disable_roles_acquisition(obj=None):
+    """Disable acquisition of local roles on given obj.
+    Set __ac_local_roles_block__ = 1 on obj.
+
+    :param obj: [required] Context object to block the acquisition on.
+    :type obj: Content object
+    :Example: :ref:`content_disable_roles_acquisition_example`
+    """
+    plone_utils = portal.get_tool('plone_utils')
+    plone_utils.acquireLocalRoles(obj, status=0)
+
+
+@required_parameters('obj')
+def enable_roles_acquisition(obj=None):
+    """Enable acquisition of local roles on given obj.
+    Set __ac_local_roles_block__ = 0 on obj.
+
+    :param obj: [required] Context object to enable the acquisition on.
+    :type obj: Content object
+    :Example: :ref:`content_enable_roles_acquisition_example`
+    """
+    plone_utils = portal.get_tool('plone_utils')
+    plone_utils.acquireLocalRoles(obj, status=1)
+
+
 @required_parameters('name', 'context', 'request')
 def get_view(name=None, context=None, request=None):
     """Get a BrowserView object.
