@@ -594,6 +594,17 @@ class TestPloneApiContent(unittest.TestCase):
         assert container['bargains']['item'].aq_base is bargain.aq_base
         assert container['products']['item']
 
+    def test_copy_same_id(self):
+        obj = self.contact
+
+        # Using the same id should fail
+        from plone.api.exc import InvalidParameterError
+        with self.assertRaises(InvalidParameterError):
+            api.content.copy(obj, obj.__parent__, obj.id)
+
+        # Using safe_id=True should work
+        api.content.copy(obj, obj.__parent__, obj.id, safe_id=True)
+
     def test_delete_constraints(self):
         """Test the constraints for deleting content."""
 
@@ -923,6 +934,34 @@ class TestPloneApiContent(unittest.TestCase):
             'internally_published',
         )
 
+    def test_diable_roles_acquisition(self):
+        """ Test disabling local roles acquisition.
+        """
+        # This should fail because an content item is mandatory
+        from plone.api.exc import MissingParameterError
+        with self.assertRaises(MissingParameterError):
+            api.content.disable_roles_acquisition()
+
+        api.content.disable_roles_acquisition(obj=self.blog)
+        blog_ac_flag = getattr(self.blog, '__ac_local_roles_block__', None)
+        self.assertTrue(blog_ac_flag)
+
+    def test_enable_roles_acquisition(self):
+        """ Test enabling local roles acquisition.
+        """
+        # This should fail because an content item is mandatory
+        from plone.api.exc import MissingParameterError
+        with self.assertRaises(MissingParameterError):
+            api.content.enable_roles_acquisition()
+
+        # As __ac_local_roles_block__ is None by default, we have to set it,
+        # before we can test the enabling method.
+        self.blog.__ac_local_roles_block__ = 1
+
+        api.content.enable_roles_acquisition(obj=self.blog)
+        blog_ac_flag = getattr(self.blog, '__ac_local_roles_block__', None)
+        self.assertFalse(blog_ac_flag)
+
     def test_get_view_constraints(self):
         """Test the constraints for deleting content."""
         from plone.api.exc import MissingParameterError
