@@ -8,7 +8,6 @@ from plone.api.tests.base import INTEGRATION_TESTING
 from plone.app.testing import TEST_USER_ID
 
 import AccessControl
-import Globals
 import unittest
 
 
@@ -51,7 +50,7 @@ class HasProtectedMethods(SimpleItem):
         pass
 
 
-Globals.InitializeClass(HasProtectedMethods)
+AccessControl.class_init.InitializeClass(HasProtectedMethods)
 
 
 class TestPloneApiEnv(unittest.TestCase):
@@ -71,8 +70,8 @@ class TestPloneApiEnv(unittest.TestCase):
 
         # Roles need to be created by name before we can assign permissions
         # to them or grant them to users.
-        for role in ('Member', 'VIP', 'Manager'):
-            portal._addRole(role)
+        # 'Member' and 'Manager' already exist by default, we need to add 'VIP'
+        portal._addRole('VIP')
 
         for permission, roles in role_mapping:
             portal.manage_permission(permission, roles, 1)
@@ -389,12 +388,13 @@ class TestPloneApiEnv(unittest.TestCase):
             api.env.adopt_roles()
 
     def test_debug_mode(self):
-        """Tests that returned value is the same as Globals.DevelopmentMode."""
+        """Tests that the retured value is the same as
+        getConfiguration.debug_mode."""
         from plone.api.env import debug_mode
-        import Globals
-        Globals.DevelopmentMode = True
+        from App.config import getConfiguration
+        getConfiguration().debug_mode = True
         self.assertEqual(debug_mode(), True)
-        Globals.DevelopmentMode = False
+        getConfiguration().debug_mode = False
         self.assertEqual(debug_mode(), False)
 
     def test_test_mode(self):
